/**
 * rot3_ticks.s
 * Pisca-pisca de um led verde
 * Projeto de um gerador de sinais periodicos em GNU Assembly (GAS)
 * Cdigo utilizado na disciplina EA871 - Primeiro semestre de 2022
 * Autora: Wu Shin Ting
 * Data:  10/07/2021
 *
 * Nota 1: Doxygen nao suporta documentacao de codigos em assembly.
 * Nota 2: ARM e Thumb Quick Reference Card: https://documentation-service.arm.com/static/5ed66080ca06a95ce53f932d?token=
 * Nota 3: Sinopse de cada instrucao: ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/ARMv6-M.pdf (A6) 
 */
 	.equ	NUM_ITERACOES,   4
 	
	.text                             @ monta no segmento de instrucoes
	.align	2						  @ alinha os enderecos em multiplos de 2 bytes 
	.global	main					  @ nome <main> qualificado como global
	.type main function               @ <main> eh uma funcao 

# Estimativa do intervalo de tempo com base no ciclo de instrucoes
# Frequencia do clock do nucleo e barramento do microcontrolador: 20,97152MHz
# Informacoes em:
# https://www.dca.fee.unicamp.br/cursos/EA871/references/ARM/Cortex-M0+.pdf - Secao 3.3
espera:								  @ <delay>
	push {r0,r2,r3,r7,lr}			  @ salvar [r0, r2, r3, r7, pc]
	sub sp,sp,#4
	add r7,sp,#0
	str r0,[r7,#0]
	
iteracao:
	mov	r2, #NUM_ITERACOES			  @ carregue a qtde de iteracoes
laco: 
	mov		r3, #5     				  @ operacoes para "gastar" tempo
	orr		r3, r0
	and		r3, r0     
	lsr		r3, #1     
	asr		r3, #1    
	
	sub 	r2, #1					 
	cmp 	r2, #0
	bne 	laco					  @ condicional NE (nao eh igual a zero) para desvio
									  @ enquanto nao atingir 0, PC <- <iteracao>
	rev 	r3,r3 
	lsl     r3,#0 
	sub 	r0,#1
	cmp 	r0, #0
	bne		iteracao
	
	mov 	sp,r7						  @ restaura so
	add 	sp,sp,#4                      @ desempilhe
	pop 	{r0,r2,r3,r7,pc}			  @ desempilhar para os registradores na sequencia r0->r2->r3->r7->pc
									   
main:								  @ <main>
	push {r0, r1, r2, r3, lr}

	#  SIM_SCGC5 = SIM_SCGC5 | (1<<13);  
	ldr 	r3,SIM_SCGC5 			  @ R3 <- [SIM_SCGC5]
	ldr 	r2,[r3]				      @ R2 <- [R3]
	movs 	r1,#128					  @ mova fonte para destino com atualizacao das flags de condicoes (128=80h)
									  @ R1 <- 0b1000 0000
	lsl 	r1,r1,#6				  @ deslocamento logico para esquerda 
									  @ R1 (0b1 0000 0000 0000)<- [R1] << 6 
	orr 	r2,r1					  @ OR logico entre fonte e destino (seta bit 13 de SIM_SCGC5 em 1)
									  @ R2 <- [R2] | [R1]
	str 	r2,[r3]                   @ [R3] <- [R2]
	#  PORTE_PCR20 = PORTE_PCR20 & 0xFFFFF8FF; (zerar os 3 bits)
	ldr 	r3,PORTE_PCR20 			  @ R3 <- [PORTE_PCR20]
	ldr 	r1,[r3]				      @ R1 <- [R3]
	ldr 	r2,RESET_MUX			  @ R2 <- [RESET_MUX]
	and 	r2,r1					  @ AND logico entre fonte e destino
									  @ R2 <- [R2] & [R1]
	#  PORTE_PCR20 = PORTE_PCR20 | 0x00000100; (setar 1 no bit 8)
	movs 	r1,#128					  @ R1 <- 128 (0b1000 0000)
	lsl 	r1,r1,#1			      @ R1 (0b1 0000 0000) < [R1] << 1
	orr 	r2,r1					  @ R2 <- [R2] | [R1]	
	str 	r2,[r3]			    	  @ [R3] <- R2
	#  GPIOE_PDDR  = GPIOE_PDDR  | (1<<20);   (setar 1 no obit 20)
	ldr 	r3,GPIOE_PDDR 			  @ R3 <- [GPIOE_PDDR]
	ldr 	r2,[r3]		    		  @ R2 <- [R3]
	movs 	r1,#128					  @ R1 <- 128 (0b1000 0000)
	lsl 	r1,r1,#13				  @ R1 (0b1 0000 0000 0000 0000 0000) < [R1] << 13
	orr 	r2,r1					  @ R2 <- [R2] | [R1]	
	str 	r2,[r3]			    	  @ [R3] <- R2
loop:								  
	#  GPIOE_PSOR = (1<<20); 
	ldr 	r3,GPIOE_PSOR 			  @ R3 <- [GPIOE_PSOR]
	movs 	r2,#128					  @ R2 <- 128
	lsl 	r2,r2,#13				  @ R2 <- [R2] << 13
	str 	r2,[r3]			     	  @ [R3] <- R2
	#  delay(10000);       
	ldr 	r0,COUNT				  @ R0 <- [COUNT]
	bl 	espera          				  @ salve [PC+2] e desvie
									  @ PC <- <delay> 
	#  GPIOE_PCOR = (1<<20); 
	ldr 	r3,GPIOE_PCOR 			  @ R3 <- [GPIOE_PCOR]
	movs 	r2,#128					  @ R2 <- 128
	lsl 	r2,r2,#13				  @ R2 <- [R2] << 13
	str 	r2,[r3]			    	  @ [R3] <- R2
	#  delay(10000);       
	ldr 	r0,COUNT				  @ R0 <- [COUNT] (primeiro argumento de espera)
	bl 	espera 						  @ PC <- <espera>	
		
	#  retorna para incio do lao
	b 	loop						  @ desvie para endereco
									  @ PC <- <loop>

	pop {r0, r1, r2, r3, pc}

	.align 2							@ alinha os endereco em multiplos de 4 bytes
SIM_SCGC5: @ Endereo do SIM_SCGC5
	.word 0x40048038
PORTE_PCR20: @ Endereo do PORTE_PCR20
	.word 0x4004D050
GPIOE_PDDR: @ Endereo do GPIOE_PDDR
	.word 0x400FF114
GPIOE_PSOR: @ Endereo do GPIOE_PSOR
	.word 0x400FF104
GPIOE_PCOR: @Endereo do GPIOE_PCOR
	.word 0x400FF108
	
RESET_MUX:
	.word 0xFFFFF8FF
COUNT:
	.word 1000                                @ multiplos de tempo-base do delay    
												  
												  	
	.end

